@@ -4,12 +4,12 @@ LDTP v2 Core Combo box.
 @author: Eitan Isaacson <eitan@ascender.com>
 @author: Nagappan Alagappan <nagappan@gmail.com>
 @copyright: Copyright (c) 2009 Eitan Isaacson
-@copyright: Copyright (c) 2009-11 Nagappan Alagappan
+@copyright: Copyright (c) 2009-12 Nagappan Alagappan
 @license: LGPL
 
 http://ldtp.freedesktop.org
 
-This file may be distributed and/or modified under the terms of the GNU General
+This file may be distributed and/or modified under the terms of the GNU Lesser General
 Public License version 2 as published by the Free Software Foundation. This file
 is distributed without any warranty; without even the implied warranty of 
 merchantability or fitness for a particular purpose.
@@ -413,8 +413,8 @@ class ComboBox(Utils, LayeredPane):
 
     def getallitem(self, window_name, object_name):
         """
-        Select combo box item
-        
+        Get all combo box item
+
         @param window_name: Window name to type in, either full name,
         LDTP's name convention, or a Unix glob.
         @type window_name: string
