@@ -1,15 +1,16 @@
+# -*- coding: utf-8 -*-
 """
 LDTP v2 Core.
 
 @author: Eitan Isaacson <eitan@ascender.com>
 @author: Nagappan Alagappan <nagappan@gmail.com>
 @copyright: Copyright (c) 2009 Eitan Isaacson
-@copyright: Copyright (c) 2009-11 Nagappan Alagappan
+@copyright: Copyright (c) 2009-12 Nagappan Alagappan
 @license: LGPL
 
 http://ldtp.freedesktop.org
 
-This file may be distributed and/or modified under the terms of the GNU General
+This file may be distributed and/or modified under the terms of the GNU Lesser General
 Public License version 2 as published by the Free Software Foundation. This file
 is distributed without any warranty; without even the implied warranty of 
 merchantability or fitness for a particular purpose.
@@ -24,7 +25,8 @@ import locale
 import subprocess
 try:
   # If we have gtk3+ gobject introspection, use that
-  from gi.repository import Wnck as wnck, Gtk as gtk, GObject as gobject
+  from gi.repository import Wnck as wnck
+  from gi.repository import Gtk as gtk
   gtk3 = True
 except:
   # No gobject introspection, use gtk2 libwnck
@@ -77,19 +79,27 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
     def __del__(self):
         if '_events' in dir(self):
             # De-register all registered events
+          try:
             pyatspi.Registry.deregisterEventListener(self._event_cb, *self._events)
             pyatspi.Registry.deregisterEventListener(self._registered_event_cb,
                                                      *self._registered_events)
+          except AttributeError:
+            # Handle exception during cleanup
+            pass
         for key in self._process_stats.keys():
             # Stop all process monitoring instances
             self._process_stats[key].stop()
 
     def _registered_event_cb(self, event):
+      try:
         if event and event.source and event.type:
             abbrev_role, abbrev_name, label_by = self._ldtpize_accessible( \
                 event.source)
             window_name = u'%s%s' % (abbrev_role, abbrev_name)
             self._callback_event.append(u"%s-%s" % (event.type, window_name))
+      except:
+        if self._ldtp_debug:
+          print traceback.format_exc()
 
     def _registered_kb_event_cb(self, event):
         if not event:
@@ -108,6 +118,7 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
                                                             event.modifiers))
 
     def _event_cb(self, event):
+      try:
         if event and event.type == "window:create" and event.source:
             for window in self._callback:
                 if window and self._match_name_to_acc(window, event.source):
@@ -124,6 +135,9 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
             if win_name in self._window_uptime:
                 self._window_uptime[win_name].append( \
                     time.strftime("%Y %m %d %H %M %S"))
+      except:
+        if self._ldtp_debug:
+          print traceback.format_exc()
 
     def getapplist(self):
         """
@@ -177,9 +191,11 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
 
     def handletablecell(self):
         self._handle_table_cell = True
+        return 1
 
     def unhandletablecell(self):
         self._handle_table_cell = False
+        return 1
 
     def delaycmdexec(self, delay = None):
         """
@@ -710,7 +726,11 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
         _current_state = _state.getStates()
         _obj_states = []
         for state in _current_state:
-            _obj_states.append(self._state_names[state.real])
+            if self._state_names[state.real] == '':
+               val = self._old_state_names[state]
+            else:
+               val = self._state_names[state]
+            _obj_states.append(val)
         return _obj_states
 
     def hasstate(self, window_name, object_name, state, guiTimeOut = 0):
@@ -734,7 +754,8 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
                 ObjectExistsWaiter(window_name, object_name, guiTimeOut, state)
             return int(waiter.run())
         except:
-            pass
+          if self._ldtp_debug:
+            print traceback.format_exc()
         return 0
 
     def grabfocus(self, window_name, object_name):
@@ -863,22 +884,6 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
         """
         return self.verifycheck(window_name, object_name)
 
-    def verifytoggled(self, window_name, object_name):
-        """
-        Verify toggle item toggled.
-        
-        @param window_name: Window name to look for, either full name,
-        LDTP's name convention, or a Unix glob.
-        @type window_name: string
-        @param object_name: Object name to look for, either full name,
-        LDTP's name convention, or a Unix glob. 
-        @type object_name: string
-
-        @return: 1 on success 0 on failure.
-        @rtype: integer
-        """
-        return self.verifycheck(window_name, object_name)
-
     def verifypushbutton(self, window_name, object_name):
         """
         Verify whether the object is push button or not.
@@ -1139,6 +1144,7 @@ class Ldtpd(Utils, ComboBox, Table, Menu, PageTabList,
             if child_name and not role and \
                     self._match_name_to_appmap(child_name, obj):
                 matches.append(name)
+            # When role and child_name args are passed
             if role and child_name and obj['class'] == role and \
                     self._match_name_to_appmap(child_name, obj):
                 matches.append(name)
