@@ -7,7 +7,7 @@ Base sequence step class and some derivatives.
 
 http://ldtp.freedesktop.org
 
-This file may be distributed and/or modified under the terms of the GNU General
+This file may be distributed and/or modified under the terms of the GNU Lesser General
 Public License version 2 as published by the Free Software Foundation. This file
 is distributed without any warranty; without even the implied warranty of 
 merchantability or fitness for a particular purpose.
@@ -41,7 +41,10 @@ class SequenceStep(gobject.GObject):
     '''
     Initialize L{SequenceStep}.
     '''
-    self.__gobject_init__()
+    try:
+      super(SequenceStep, self).__init__()
+    except:
+      self.__gobject_init__()
     self.done = False
 
   def stepDone(self):
