@@ -1,15 +1,16 @@
+# -*- coding: utf-8 -*-
 '''
 LDTP v2 utils.
 
 @author: Eitan Isaacson <eitan@ascender.com>
 @author: Nagappan Alagappan <nagappan@gmail.com>
 @copyright: Copyright (c) 2009 Eitan Isaacson
-@copyright: Copyright (c) 2009-11 Nagappan Alagappan
+@copyright: Copyright (c) 2009-12 Nagappan Alagappan
 @license: LGPL
 
 http://ldtp.freedesktop.org
 
-This file may be distributed and/or modified under the terms of the GNU General
+This file may be distributed and/or modified under the terms of the GNU Lesser General
 Public License version 2 as published by the Free Software Foundation. This file
 is distributed without any warranty; without even the implied warranty of 
 merchantability or fitness for a particular purpose.
@@ -22,6 +23,7 @@ Headers in this file shall remain intact.
 import os
 import re
 import time
+import fnmatch
 import logging
 import pyatspi
 import threading
@@ -175,6 +177,7 @@ class Utils:
         self._states_old = {}
         self._logger = logger
         self._state_names = {}
+        self._old_state_names = {}
         self._window_uptime = {}
         self._callback_event = []
         self._delaycmdexec = None
@@ -183,6 +186,8 @@ class Utils:
         self._custom_logger = _custom_logger
         self._desktop = pyatspi.Registry.getDesktop(0)
         self._ldtp_debug = os.environ.get('LDTP_DEBUG', None)
+        # Initialize atspi2 version to False
+        self._atspi2_ver = False
         if Utils.cached_apps is None:
             pyatspi.Registry.registerEventListener(
                 self._on_window_event, 'window')
@@ -192,10 +197,12 @@ class Utils:
                 self._on_window_event, 'window:destroy')
             # Notify on any changes in all windows, based on this info,
             # its decided, whether force_remap is required or not
-            pyatspi.Registry.registerEventListener(self._obj_changed, 
-                                                   'object:children-changed')
-            pyatspi.Registry.registerEventListener(
-                self._obj_changed, 'object:property-change:accessible-name')
+            # Commenting the following lines of code
+            # as it sucks the execution time in at-spi2
+            #pyatspi.Registry.registerEventListener(self._obj_changed, 
+            #                                       'object:children-changed')
+            #pyatspi.Registry.registerEventListener(
+            #    self._obj_changed, 'object:property-change:accessible-name')
 
             Utils.cached_apps = list()
             if lazy_load:
@@ -220,6 +227,8 @@ class Utils:
             # b.g.o Bug#654683
             self._states[str(state)] = state
             # Ignore STATE_ string for LDTPv1 compatibility
+            self._old_state_names[state] = \
+                state.__repr__().lower().partition("state_")[2]
             self._state_names[state] = \
                 str(state).lower().partition("state_")[2]
         return self._states
@@ -230,7 +239,12 @@ class Utils:
         as the children have changed
         """
         if self._ldtp_debug:
-            print event, event.type, event.source, event.source.parent
+            try:
+                print event, event.type, event.source, event.source.parent
+            except:
+                # With at-spi2, sometimes noticed exception
+                # ignore exception, as we just use them for debugging
+                pass
         if not self.cached_apps:
             # If not initialized as list, don't process further
             return
@@ -253,56 +267,88 @@ class Utils:
 
     def _on_window_event(self, event):
         if self._ldtp_debug:
-            print event, event.type, event.source, event.source.parent
-        # Proceed only for window destry and deactivate event
-        if event and (event.type == "window:destroy" or \
-                          event.type == "window:deactivate") and \
-                          event.source:
-            abbrev_role, abbrev_name, label_by = self._ldtpize_accessible( \
-                event.source)
-            # LDTPized name
-            win_name = u'%s%s' % (abbrev_role, abbrev_name)
-            # Window title is empty
-            if abbrev_name == '':
-                for win_name in self._appmap.keys():
-                    # When window doesn't have a title, destroy all the
-                    # window info from appmap, which doesn't haven't title
-                    if re.search('%s\d*$' % abbrev_role, win_name, re.M | re.U):
-                        del self._appmap[win_name]
-            else:
-                for name in self._appmap.keys():
-                    # When multiple window have same title, destroy all the
-                    # window info from appmap, which have same title
-                    if re.search('%s%s\d*$' % (abbrev_role, abbrev_name),
-                                 win_name, re.M | re.U) or \
-                                 re.search('%s%s*$' % (abbrev_role, abbrev_name),
-                                           win_name, re.M | re.U):
-                        del self._appmap[name]
+            try:
+                print event, event.type, event.source, event.source.parent
+            except:
+                # With at-spi2, sometimes noticed exception
+                # ignore exception, as we just use them for debugging
+                pass
+        try:
+            # Proceed only for window destry and deactivate event
+            if event and (event.type == "window:destroy" or \
+                              event.type == "window:deactivate") and \
+                              event.source:
+                abbrev_role, abbrev_name, label_by = self._ldtpize_accessible( \
+                    event.source)
+                # LDTPized name
+                win_name = u'%s%s' % (abbrev_role, abbrev_name)
+                # Window title is empty
+                if abbrev_name == '':
+                    for win_name in self._appmap.keys():
+                        # When window doesn't have a title, destroy all the
+                        # window info from appmap, which doesn't haven't title
+                        if re.search('%s\d*$' % abbrev_role, win_name, re.M | re.U):
+                            del self._appmap[win_name]
+                else:
+                    for name in self._appmap.keys():
+                        # When multiple window have same title, destroy all the
+                        # window info from appmap, which have same title
+                        if re.search('%s%s\d*$' % (abbrev_role, abbrev_name),
+                                     win_name, re.M | re.U) or \
+                                     re.search('%s%s*$' % (abbrev_role, abbrev_name),
+                                               win_name, re.M | re.U):
+                                     del self._appmap[name]
+                return
+            cache = True
+            if not self.cached_apps:
+                # If not initialized as list, don't process further
+                return
+            for app in self.cached_apps:
+                if event.host_application == app[0]:
+                    # Application already in cached list
+                    cache = False
+                    # Force remap for this application, as some object is
+                    # either added / removed / changed
+                    index = self.cached_apps.index(app)
+                    self.cached_apps[index][1] = True
+                    if hasattr(app, 'setCacheMask'):
+                        app.setCacheMask(pyatspi.cache.ALL)
+                    break
+            if cache:
+                # If app doesn't exist in cached apps, then add it
+                # adding app and True flag as list - This flag indicates that the
+                # object in application either got added / removed
+                # so remap should be forced
+                self.cached_apps.append([event.host_application, True])
+        except:
+            if self._ldtp_debug:
+                print traceback.format_exc()
+
+    def _atspi2_workaround(self):
+        if not hasattr(pyatspi, 'Accessible'):
+            # This exist only in pyatspi2
+            # Don't do the work around
             return
-        cache = True
-        if not self.cached_apps:
-            # If not initialized as list, don't process further
-            return
-        for app in self.cached_apps:
-            if event.host_application == app[0]:
-                # Application already in cached list
-                cache = False
-                # Force remap for this application, as some object is
-                # either added / removed / changed
-                index = self.cached_apps.index(app)
-                self.cached_apps[index][1] = True
-                break
-        if cache:
-            # If app doesn't exist in cached apps, then add it
-            # adding app and True flag as list - This flag indicates that the
-            # object in application either got added / removed
-            # so remap should be forced
-            self.cached_apps.append([event.host_application, True])
+        self._atspi2_ver = True
+        for app in self._desktop:
+            # Work around for at-spi2
+            if not app: continue
+            flag = False
+            for tmpapp in self.cached_apps:
+                if app == tmpapp[0]:
+                    flag = True
+                    break
+            if hasattr(app, 'setCacheMask'):
+                app.setCacheMask(pyatspi.cache.ALL)
+            # App already in list, don't add again
+            if flag: continue
+            self.cached_apps.append([app, True])
 
     def _list_apps(self):
         """
         List all the applications
         """
+        self._atspi2_workaround()
         for app in self.cached_apps:
             if not app: continue
             yield app
@@ -311,6 +357,7 @@ class Utils:
         """
         List all the windows that are currently open
         """
+        self._atspi2_workaround()
         for app in self.cached_apps:
             if not app or not app[0]: continue
             try:
@@ -338,7 +385,7 @@ class Utils:
         try:
             # Get accessible relation set
             rel_set = acc.getRelationSet()
-        except LookupError:
+        except:
             rel_set = None
         if rel_set:
             for i, rel in enumerate(rel_set):
@@ -361,7 +408,11 @@ class Utils:
                         if self._ldtp_debug:
                             print traceback.format_exc()
                         continue
-        role = acc.getRole()
+        try:
+            role = acc.getRole()
+        except:
+            # with at-spi2 noticed gi._glib.GError exception
+            role = None
         if role == pyatspi.ROLE_FRAME or role == pyatspi.ROLE_DIALOG or \
                 role == pyatspi.ROLE_WINDOW or \
                 role == pyatspi.ROLE_FONT_CHOOSER or \
@@ -375,14 +426,17 @@ class Utils:
             # all other object types
             strip = '( |:|\.|_|\n)'
         if label_acc:
-            # Priority to associated label
-            label_by = label_acc.name
+            try:
+                # Priority to associated label
+                label_by = label_acc.name
+            except:
+                label_by = ''
         # Return the role type (if, not in the know list of roles,
         # return ukn - unknown), strip the above characters from name
         # also return labely_by string
         try:
             label = re.sub(strip, '', (label_acc or acc).name)
-        except LookupError:
+        except:
             label = ''
         return abbreviated_roles.get(role, 'ukn'), \
             label, \
@@ -411,41 +465,51 @@ class Utils:
         @return: Return 0 on failure, 1 on successful match
         @rtype: integer
         """
-        if not acc:
+        if not acc or not name:
             return 0
-        if classType:
-            # Accessibility role type returns space, when multiple
-            # words exist in object type. ex: 'push button'
-            # User might mistype with multiple space, to avoid
-            # any confusion, using _. So, user will be inputing
-            # push_button
-            roleName = acc.getRoleName().replace(' ', '_')
-        else:
-            roleName = None
-        if roleName != classType:
-            # If type doesn't match, don't proceed further
+        try:
+            if classType:
+                # Accessibility role type returns space, when multiple
+                # words exist in object type. ex: 'push button'
+                # User might mistype with multiple space, to avoid
+                # any confusion, using _. So, user will be inputing
+                # push_button
+                roleName = acc.getRoleName().replace(' ', '_')
+            else:
+                roleName = None
+            if roleName != classType:
+                # If type doesn't match, don't proceed further
+                return 0
+            if acc.name and re.match(fnmatch.translate(name), acc.name, re.M | re.U):
+                # Since, type already matched and now the given name
+                # and accessibile name matched, mission accomplished
+                return 1
+        except:
+            # In at-spi2 gi._glib.GError exception is thrown
             return 0
-        if acc.name == name:
-            # Since, type already matched and now the given name
-            # and accessibile name matched, mission accomplished
-            return 1
         # Get LDTP format accessibile name
         _ldtpize_accessible_name = self._ldtpize_accessible(acc)
         # Concat object type and object name
         # ex: 'frmUnsavedDocument1-gedit' for Gedit application
         # frm - Frame, Window title - 'Unsaved Document 1 - gedit'
-        _object_name = u'%s%s' % (_ldtpize_accessible_name[0],
-                                  _ldtpize_accessible_name[1])
-        if _object_name == name:
-            # If given name equal LDTPized name format
-            return 1
+        try:
+           _object_name = u'%s%s' % (_ldtpize_accessible_name[0],
+                                     _ldtpize_accessible_name[1])
+        except UnicodeDecodeError:
+           _object_name = u'%s%s' % (_ldtpize_accessible_name[0],
+                                     _ldtpize_accessible_name[1].decode('utf-8'))
         if self._glob_match(name, acc.name):
             # If given name match object name with regexp
             return 1
         if self._glob_match(name, _object_name):
             # If given name match LDTPized name format with regexp
             return 1
-        role = acc.getRole()
+        try:
+            role = acc.getRole()
+        except:
+            # In at-spi2 acc doesn't exist
+            # which raises exception gi._glib.GError
+            return 0
         if role == pyatspi.ROLE_FRAME or role == pyatspi.ROLE_DIALOG or \
                 role == pyatspi.ROLE_WINDOW or \
                 role == pyatspi.ROLE_FONT_CHOOSER or \
@@ -582,16 +646,37 @@ class Utils:
             self.ldtpized_list[parent]['children'] = _current_children
         if not label_by:
             label_by = ''
+        key_binding = ''
+        try:
+            iaction = obj.queryAction()
+            for j in xrange(iaction.nActions):
+                if iaction.getKeyBinding(j) != '':
+                    key_binding = iaction.getKeyBinding(j)
+                    break
+        except NotImplementedError:
+            pass
+        role = obj.getRole()
+        if role == pyatspi.ROLE_FRAME or role == pyatspi.ROLE_DIALOG or \
+                role == pyatspi.ROLE_WINDOW or \
+                role == pyatspi.ROLE_FONT_CHOOSER or \
+                role == pyatspi.ROLE_FILE_CHOOSER or \
+                role == pyatspi.ROLE_ALERT or \
+                role == pyatspi.ROLE_COLOR_CHOOSER:
+            obj_index = '%s#%d' % (obj.getApplication().name,
+                                   obj.getIndexInParent())
+        else:
+            obj_index = '%s#%d' % (abbrev_role,
+                                   self.ldtpized_obj_index[abbrev_role])
         self.ldtpized_list[ldtpized_name] = {'key' : ldtpized_name,
                                              'parent' : parent,
                                              'class' : obj.getRoleName().replace(' ', '_'),
                                              'child_index' : child_index,
                                              'children' : '',
-                                             'obj_index' : '%s#%d' % (abbrev_role,
-                                                                      self.ldtpized_obj_index[abbrev_role]),
+                                             'obj_index' : obj_index,
                                              'label' : obj.name,
                                              'label_by' : label_by,
-                                             'description' : obj.description
+                                             'description' : obj.description,
+                                             'key_binding' : key_binding
                                              }
         return ldtpized_name
 
@@ -615,6 +700,7 @@ class Utils:
         self.ldtpized_list = {}
         self.ldtpized_obj_index = {}
         if not force_remap:
+            self._atspi2_workaround()
             for app in self.cached_apps:
                 try:
                     if app[0] and gui and app[0] == gui.parent and \
@@ -684,7 +770,9 @@ class Utils:
                 'Object does not have an Action interface')
         else:
             for i in xrange(iaction.nActions):
-                if re.match(action, iaction.getName(i)):
+                if self._ldtp_debug:
+                    print iaction.getName(i)
+                if re.match(action, iaction.getName(i), re.I):
                     iaction.doAction(i)
                     return
             raise LdtpServerException('Object does not have a "%s" action' % action)
@@ -756,12 +844,27 @@ class Utils:
                 return gui, name
 
             # Search with LDTP appmap format
+            if window_name.find('#') != -1:
+                obj_index = '%s#%d' % (gui.getApplication().name,
+                                       gui.getIndexInParent())
+                if self._ldtp_debug:
+                    print 'Window name has #', window_name, obj_index
+                if window_name == obj_index:
+                    if self._ldtp_debug:
+                        print 'Window found', gui, name
+                    return gui, name
             if window_name == name:
+                if self._ldtp_debug:
+                    print 'Window found', gui, name
                 return gui, name
             if self._glob_match(window_name, name):
+                if self._ldtp_debug:
+                    print 'Window found', gui, name
                 return gui, name
             if self._glob_match(re.sub(' ', '', window_name),
                                 re.sub(' ', '', name)):
+                if self._ldtp_debug:
+                    print 'Window found', gui, name
                 return gui, name
         return None, None
 
