@@ -4,12 +4,12 @@ LDTP v2 Core Value.
 @author: Eitan Isaacson <eitan@ascender.com>
 @author: Nagappan Alagappan <nagappan@gmail.com>
 @copyright: Copyright (c) 2009 Eitan Isaacson
-@copyright: Copyright (c) 2009-11 Nagappan Alagappan
+@copyright: Copyright (c) 2009-12 Nagappan Alagappan
 @license: LGPL
 
 http://ldtp.freedesktop.org
 
-This file may be distributed and/or modified under the terms of the GNU General
+This file may be distributed and/or modified under the terms of the GNU Lesser General
 Public License version 2 as published by the Free Software Foundation. This file
 is distributed without any warranty; without even the implied warranty of 
 merchantability or fitness for a particular purpose.
@@ -110,7 +110,7 @@ class Value(Utils):
             obj = self._get_object(window_name, object_name)
             valuei = obj.queryValue()
 
-            if valuei.currentValue == data:
+            if valuei.currentValue == float(data):
                 return 1
         except:
             pass
